#ifndef CW_FACEDETTRACK_H_
#define CW_FACEDETTRACK_H_
#include "CWFaceDetTrackDef.h"

//#define CW_FACE_DET_TRACK_EXPORTS_C
#ifdef CW_FACE_DET_TRACK_EXPORTS_C
#define CW_DET_TRACK_C_API  __declspec(dllexport)
#else
#define CW_DET_TRACK_C_API
#endif

#if defined (__cplusplus)
extern "C" {
#endif	

	/*** ================================必读重要事项======================================
	1、正式发布的版本的时候一定要将setLog 和 setLogAndSaveImage 的日志关闭！！！！否则将严重影响性能！！！！！！！！！！！！
	1、正式发布的版本的时候一定要将setLog 和 setLogAndSaveImage 的日志关闭！！！！否则将严重影响性能！！！！！！！！！！！！
	2、每次结束一个人的动作活体的所有状态时（即完成动作状态返回准备状态），需要调用resetLivenessTarget。动作状态后面还是动作状态就不需要调用。
	3、建议：选择动作时，建议从眨眼和张嘴两个动作里面必选其一，其他的可以随机组合,当前版本这两个动作效果更好。
	===========================================================================================
	***/

	/**
	*
	* 根据传入的授权码及模型文件创建人脸活体检测器.
	* 输入：
	*		pLicence               - 授权码.
	*      modelMode              - 加载模型方式.
	*		pFaceDetectFile        - 人脸检测模型文件.
	*		pFaceKeyPtDetFile      - 关键点检测模型文件.
	*		pFaceKeyPtTrackFile    - 关键点跟踪模型文件.
	*		pFaceQualityFile       - 人脸质量评估模型文件.
	*		pFaceLivenessFile      - 活体模型文件.
	* 输出：
	*      errCode - 成功返回CW_FACE_OK，失败返回其他.
	* 返回值：
	*		检测器句柄 - 成功返回句柄，失败返回0.
	*/
	CW_DET_TRACK_C_API
		void* cwCreateDetector(cw_face_errcode_t* errCode,
			cw_model_mode_t modelMode,
			const char* pLicence,
			const char* pFaceDetectFile = 0,
			const char* pFaceKeyPtDetFile = 0,
			const char* pFaceKeyPtTrackFile = 0,
			const char* pFaceQualityFile = 0,
			const char* pFaceLivenessFile = 0);

	/**
	* 释放检测器
	* 输入：
	*		pDetector - 检测器句柄
	* 输出：
	*      无
	* 返回值：
	*		无
	*/
	CW_DET_TRACK_C_API
		void cwReleaseDetector(void *&pDetector);

	/**
	* 获取检测器参数
	* 输入：
	*		pDetector - 检测器句柄
	* 输出：
	*      param - 检测器参数
	* 返回值：
	*		cw_face_errcode_t - 成功返回CW_FACE_OK，失败返回其他
	*/
	CW_DET_TRACK_C_API
		cw_face_errcode_t cwGetParam(void* pDetector, cw_face_param_t* param);


	/**
	* 设置检测器参数（必须先调用cwGetParam(pDet,&params)再使用此函数）
	* 输入：
	*      pDetector - 检测器句柄
	*		param - 参数
	* 输出：
	*      无
	* 返回值：
	*		cw_face_errcode_t - 成功返回CW_FACE_OK，失败返回其他
	*/
	CW_DET_TRACK_C_API
		cw_face_errcode_t cwSetParam(void* pDetector, const cw_face_param_t* param);


	/**
	* 人脸检测跟踪接口
	* 输入：
	*      pDetector - 检测器句柄
	*		frameImg - 被检测图像
	*		buffLen - 存放检测结果pFaceBuffer数组的元素个数
	*		op - 操作码
	*       to_next_stage 的结果反馈如下，括号里为可能返回的阶段:
	*           16:     准备阶段检测到口罩(准备)
	*	    	15:		准备阶段检查到墨镜(准备)
	*	    	14: 	准备阶段检测到闭眼(准备)
	*			13:		准备阶段检测到张嘴(准备)
	*			12:		
	*			11:	
	*			10:		人脸被遮挡(准备、张嘴、眨眼)
	*			9:		
	*			8:		人脸不在屏幕中间(准备、张嘴、眨眼)
	*			7:		人脸太亮(准备)
	*			6:		人脸太暗(准备)
	*			5:		人脸晃动(准备)
	*			4:		人脸角度没有正对屏幕(准备)
	*			3:		人脸太近(所有)
	*			2:		人脸太远(所有)
	*			1:		通过测试(所有)
	*			0:		等待(所有)
	*			-1:		回放攻击(所有)
	*			-2:		面具攻击(所有)
	*			-3:		
	*	    	-4:		黑白图片攻击(所有)
	*	    	-5:		边框攻击(所有)
	*	    	-6:		
	*	    	-7:		换人攻击(张嘴、眨眼、左右转)
	* 输出：
	*		pFaceBuffer - 存放检测结果的数组
	*		nFace - 实际被检测和跟踪到的人脸数
	* 返回值：
	*		cw_face_errcode_t - 成功返回CW_FACE_OK，失败返回其他
	* 说明：当前准备阶段的to_next_stage可能返回上面所有的码。 
	*/
	CW_DET_TRACK_C_API
		cw_face_errcode_t cwFaceDetectTrack(void* pDetector,
			cw_image_t* frameImg,
			cw_face_t* pFaceBuffer,
			int buffLen,
			int* nFace,
			int* to_next_stage,
			int op = CW_FACE_DET);



	/**
	* 获取最佳人脸图像和关键点信息
	* return：
	*		存储人脸图像和关键点的结构体，长度为4帧，传给后端的可以用两帧
	*
	*/
	CW_DET_TRACK_C_API
		cw_face_errcode_t cwGetLivingImageSample(void* pDetector
			, cw_living_image_frame_t* livingImageBuffer);


	//	/**
	//	 * 版本信息函数
	CW_DET_TRACK_C_API
		const char* cwGetVersionInfo(void* pDetector);


	//	/**
	//  重要：每完成或中断一个活体流程，重置活体状态时必须调用！！！！！！！！！！
	//	 * 每完成或中断一个活体流程，重置活体状态
	//	 * 输入：
	//	 *		检测器
	//	 * 输出：
	//	 *      无
	//	 */
	CW_DET_TRACK_C_API
		void resetLivenessTarget(void* pDetector);

	//	/**
	//	 * 活体log开关（存图版本）
	//   * 输入： pDetector： 检测器
	//   *        openLog ： 是否打开日志
	//   *        logPath：日志存储路径，需要文件夹存在，如/data/local/tmp/。将会生成日志文件夹/data/local/tmp/cloudwalk
	//   *        saveImage: 是否开启存图。正常需要开启。
	//   * 日志保护机制：当完成999人次的动作活体时，将会自动关闭日志。
	//   * 日志仅为debug使用，正式使用时请一定要将openLog设置成false，或者直接不使用日志。即关闭日志！！！！！！！。否则会严重影响性能。
	//   * 日志仅为debug使用，正式使用时请一定要将openLog设置成false，或者直接不使用日志。即关闭日志！！！！！！！。否则会严重影响性能。
	//   * 日志仅为debug使用，正式使用时请一定要将openLog设置成false，或者直接不使用日志。即关闭日志！！！！！！！。否则会严重影响性能。
	CW_DET_TRACK_C_API
		void setLogAndSaveImage(void* pDetector, bool openLog, char *logPath, bool saveImage = true);

	//	/**
	//	 * 活体log开关
	CW_DET_TRACK_C_API
		void setLog(void* pDetector, bool openLog);


	//	/**
	//	 * 获取log信息
	CW_DET_TRACK_C_API
		int getLogInfo(void* pDetector, char* buf, int nBufSize);


	//	/**
	//	 * 最佳人脸图片防攻击检测
	//	 *      无
	//   * 返回：
	//   *      0: 真人
	//   *      负数：攻击
	//	 */
	CW_DET_TRACK_C_API
		int cwVerifyBestImg(void* pDetector);

#if defined (__cplusplus)
}
#endif

#endif   // CW_FACEDETTRACK_H_
