#include <string.h>
#include "Common.h"

long clock_time() {//ms
    struct timespec now;
    clock_gettime(CLOCK_REALTIME , &now);//CLOCK_MONOTONIC
    return (now.tv_sec * 1000000000LL + now.tv_nsec) / 1000000;
}

std::string jstringToPChar(JNIEnv *env, jstring js, const char *encode/*="gbk"*/) {
    jclass clsString = env->FindClass("java/lang/String");
    jstring jencoding = env->NewStringUTF(encode);
    jmethodID midgetBytes = env->GetMethodID(clsString, "getBytes", "(Ljava/lang/String;)[B");
    jbyteArray barr = (jbyteArray) env->CallObjectMethod(js, midgetBytes, jencoding);
    int len = env->GetArrayLength(barr);
    jbyte *ba = new jbyte[len];
    env->GetByteArrayRegion(barr, 0, len, ba);
    std::string str;
    if (len > 0) {
        str = std::string((const char *) ba, 0, len);
    }
    if (ba) {
        delete[] ba;
        ba = NULL;
    }
    env->DeleteLocalRef(jencoding);
    env->DeleteLocalRef(barr);
    env->DeleteLocalRef(clsString);
    return str;
}

jstring cstrToJstring(JNIEnv *env, const char *instr, const char *encode/*="gbk"*/) {
    jclass clsString = env->FindClass("java/lang/String");
    jmethodID midStringInit = env->GetMethodID(clsString, "<init>", "([BLjava/lang/String;)V");
    const int inlen = strlen(instr);
    jstring jencoding = env->NewStringUTF(encode);
    jbyteArray bytes = env->NewByteArray(inlen);
    env->SetByteArrayRegion(bytes, 0, inlen, (jbyte *) instr);
    jstring js = (jstring) env->NewObject(clsString, midStringInit, bytes, jencoding);
    env->DeleteLocalRef(jencoding);
    env->DeleteLocalRef(bytes);
    env->DeleteLocalRef(clsString);
    return js;
}

void initHashMap(JNIEnv *env, jclass& class_hashmap, jmethodID* hashmap_init, jmethodID* hashMap_put, jmethodID* hashMap_get) {
    class_hashmap = env->FindClass("java/util/HashMap");
    if (hashmap_init)
        *hashmap_init = env->GetMethodID(class_hashmap, "<init>", "()V");
    if (hashMap_put)
        *hashMap_put = env->GetMethodID(class_hashmap, "put", "(Ljava/lang/Object;Ljava/lang/Object;)Ljava/lang/Object;");
    if (hashMap_get)
        *hashMap_get = env->GetMethodID(class_hashmap, "get", "(Ljava/lang/Object;)Ljava/lang/Object;");
}

void putMapOjbect(JNIEnv *env, jobject hashMap, jmethodID hashMap_put, const char *key, jobject obj) {
    env->CallObjectMethod(hashMap, hashMap_put, env->NewStringUTF(key), obj);
}

void putMapValue(JNIEnv *env, jobject hashMap, jmethodID hashMap_put, const char *key, const char *value) {
    env->CallObjectMethod(hashMap, hashMap_put, env->NewStringUTF(key), env->NewStringUTF(value));
}

void putMapValue(JNIEnv *env, jobject hashMap, jmethodID hashMap_put, const char *key, int value) {
    char buffer[256] = {0};
    sprintf(buffer, "%d", value);
    putMapValue(env, hashMap, hashMap_put, key, buffer);
}

void putMapValue(JNIEnv *env, jobject hashMap, jmethodID hashMap_put, const char *key, long value) {
    char buffer[256] = {0};
    sprintf(buffer, "%ld", value);
    putMapValue(env, hashMap, hashMap_put, key, buffer);
}

void putMapValue(JNIEnv *env, jobject hashMap, jmethodID hashMap_put, const char *key, float value) {
    char buffer[256] = {0};
    sprintf(buffer, "%f", value);
    putMapValue(env, hashMap, hashMap_put, key, buffer);
}

bool getMapValue(JNIEnv *env, jobject hashMap, jmethodID hashMap_get, std::string& value, const char *key) {
    jstring js = (jstring)env->CallObjectMethod(hashMap, hashMap_get, env->NewStringUTF(key));
    if (NULL!=js) {
        value = jstringToPChar(env, js);
        return true;
    }
    return false;
}

bool getMapValue(JNIEnv *env, jobject hashMap, jmethodID hashMap_get, int& value, const char *key) {
    std::string str;
    if (getMapValue(env, hashMap, hashMap_get, str, key)) {
        sscanf(str.c_str(),"%d",&value);
        return true;
    }
    return false;
}

bool getMapValue(JNIEnv *env, jobject hashMap, jmethodID hashMap_get, long& value, const char *key) {
    std::string str;
    if (getMapValue(env, hashMap, hashMap_get, str, key)) {
        sscanf(str.c_str(),"%ld",&value);
        return true;
    }
    return false;
}

bool getMapValue(JNIEnv *env, jobject hashMap, jmethodID hashMap_get, float& value, const char *key) {
    std::string str;
    if (getMapValue(env, hashMap, hashMap_get, str, key)) {
        sscanf(str.c_str(),"%f",&value);
        return true;
    }
    return false;
}