#include "NDKFaceDetTrack.h"
#include "JniHelpers/LogUtil.h"
#include "Object/FaceLivingImg.h"
#include <stdlib.h>
#include <stdio.h>
#define MAX_FACE_NUM 2 //需和CloudwalkSDK.java中maxFaceNum一致

jclass  m_NativeFaceInfoClass;//FaceInfo类
jmethodID m_CFaceInfo;//FaceInfo默认无参构造方法
jfieldID  m_NativeFaceInfoFiled;//FaceDetTrack中FaceInfo[] faceInfos字段
jfieldID m_NativeFaceDetFiled;
jfieldID m_NativeFaceNumFiled;

jint NDKFaceDetTrack::CreateDetectorFromFile(JNIEnv *env, jobject thiz,
                                             jstring pFaceDetectFile, // ?????????????
                                             jstring pFaceKeyPtFile, // ??????????????
                                             jstring pFaceKeyPtTrackFile,
                                             jstring pFaceQualityFile, // ??????????????????
                                             jstring pFaceLivenessFile,
                                             jstring pLicence,
                                             jint mode) {
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);

    JavaString jFaceDetectFile(env, pFaceDetectFile);

    JavaString jFaceKeyPtFile(env, pFaceKeyPtFile);
    JavaString jFaceKeyPtTrackFile(env, pFaceKeyPtTrackFile);
    JavaString jFaceQualityFile(env, pFaceQualityFile);
    JavaString jFaceLivenessFile(env, pFaceLivenessFile);
    JavaString jLicence(env, pLicence);

    cw_face_errcode_t err;
    std::string detFile = jFaceDetectFile.get();
    std::string KeyPtFile = jFaceKeyPtFile.get();
    std::string KeyPtTrackFile = jFaceKeyPtTrackFile.get();
    std::string QualityFile = jFaceQualityFile.get();
    std::string LivenessFile = jFaceLivenessFile.get();
    std::string Licence = jLicence.get();

    //检查模型文件是否存在
    std::vector<const char*>  modules;
    modules.push_back(detFile.c_str());
    modules.push_back(KeyPtFile.c_str());
    modules.push_back(KeyPtTrackFile.c_str());
    modules.push_back(QualityFile.c_str());
    modules.push_back(LivenessFile.c_str());

    for (int i = 0; i < modules.size(); ++i) {
        const char* fileName = modules.at(i);
        FILE* fh = fopen(fileName,"r");
        if(fh == NULL)
        {
            LOGE("module file %s not exit or not readable",fileName);
        } else{
            //LOGE("module file %s exist",fileName);
            fclose(fh);
        }
    }

    //加载了armeabi文件夹下so
    #ifdef ARMEABI_LOW_SPEED_MODE
        LOGE("*************************************");
        LOGE("   *****ARMEABI LOW SPEED MODE*****   ");
        LOGE("*************************************");
    #endif

    void *pHandle;
    long nativeDet = env->GetLongField(thiz,m_NativeFaceDetFiled);//获取Native检测器的指针

    //判断native层句柄是否已初始化,如果已初始化则销毁再创建
    if(nativeDet!=0){//已经初始化
        cwReleaseDetector(pHandle);//销毁Native检测句柄
        env->SetLongField(thiz,m_NativeFaceDetFiled,0);//设置Java层属性mNativeDet为0
    }

    pHandle = cwCreateDetector(&err,(cw_model_mode_t) mode,
                                     Licence.c_str(),
                                     detFile.c_str(),
                                     KeyPtFile.c_str(),
                                     KeyPtTrackFile.c_str(),
                                     QualityFile.c_str(),
                                     LivenessFile.c_str());

    if (pHandle == nullptr || err != CW_FACE_OK) {
        LOG_ERROR("cwCreateDetector error=%d", err);
        LOGE("cwCreateDetector error=%d", err);
        env->SetLongField(thiz,m_NativeFaceDetFiled,0);
        return err;
    }
    env->SetLongField(thiz,m_NativeFaceDetFiled,(jlong)pHandle);//设置Java层属性mNativeDet为Native检测器的指针
    return 0;
}

jint NDKFaceDetTrack::ReleaseDetector(JNIEnv *env, jobject thiz) {
    LOG_INFO("%s %d", __FUNCTION__, __LINE__);

    void *pDet =(void *) env->GetLongField(thiz,m_NativeFaceDetFiled);//获取Native检测器的指针
    if (pDet == nullptr) {
        return CW_FACE_UNINITIALIZED_ERR;
    }

    cwReleaseDetector(pDet);
    env->SetLongField(thiz,m_NativeFaceDetFiled,0);
    return CW_FACE_OK;
}


jint NDKFaceDetTrack::ResetLivenessTarget(JNIEnv *env, jobject thiz) {
    LOG_INFO("%s %d", __FUNCTION__, __LINE__);

    void *pDet =(void *) env->GetLongField(thiz,m_NativeFaceDetFiled);//获取Native检测器的指针
    if (pDet == nullptr) {
        return CW_FACE_UNINITIALIZED_ERR;
    }
    resetLivenessTarget(pDet);
    return CW_FACE_OK;
}

jint NDKFaceDetTrack::VerifyBestImg(JNIEnv *env, jobject thiz) {
    LOG_INFO("%s %d", __FUNCTION__, __LINE__);

    void *pDet =(void *) env->GetLongField(thiz,m_NativeFaceDetFiled);//获取Native检测器的指针
    if (pDet == nullptr) {
        return CW_FACE_UNINITIALIZED_ERR;
    }
    return cwVerifyBestImg(pDet);//前端活体校验
}

jint NDKFaceDetTrack::GetParam(JNIEnv *env, jobject thiz, jobject faceParam) {
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);

    void *pDet =(void *) env->GetLongField(thiz,m_NativeFaceDetFiled);//获取Native检测器的指针
    if (pDet == nullptr) {
        return CW_FACE_UNINITIALIZED_ERR;
    }

    cw_face_param_t param;
    cw_face_errcode_t err = cwGetParam(pDet, &param);
    if (err != CW_FACE_OK) {
        LOG_ERROR("cwGetParam error on handle[%d] err=%d", handle, err);
        return err;
    }


    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
    FaceParam p(env, &param);
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
    p.mapFields();
    p.toJavaObject(env, faceParam);
    return CW_FACE_OK;


}

jint NDKFaceDetTrack::SetParam(JNIEnv *env, jobject thiz, jobject faceParam) {
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
    void *pDet =(void *) env->GetLongField(thiz,m_NativeFaceDetFiled);//获取Native检测器的指针
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
    if (pDet == nullptr) {
        LOG_ERROR("GetParam can not find handle[%d]", handle);
        return CW_FACE_UNINITIALIZED_ERR;
    }
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
    cw_face_param_t param;
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);

    FaceParam p(env);
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
    p.setJavaObject(env, faceParam);
    param.roiX = p.roiX;
    param.roiY = p.roiY;
    param.maxFaceNumPerImg = p.maxFaceNumPerImg;  // 每帧最大人脸数，默认20
//    param.minSize = p.minSize;                   // 人脸尺寸范围， pc端默认(30 ~ 1000);移动端默认(100 ~ 400)
//    param.maxSize = p.maxSize;
    param.roiHeight = p.roiHeight;
    param.roiWidth = p.roiWidth;
//    param.globleDetFreq = p.globleDetFreq;          // 全局检测频率， pc端默认10；移动端默认25
//    param.minTrackedKeyptScore = p.minTrackedKeyptScore;//新增的返回去的参数
//    param.imageMode = p.imageMode;
//    param.nMissAllowed = p.nMissAllowed;
//    param.nCpuCores = p.nCpuCores;
    param.perfmonLevel = p.perfmonLevel;
//    param.minLivingBrightness = p.minLivingBrightness;
//    param.bCollectFrmsLiving = p.bCollectFrmsLiving;
//
//    param.bSaveImgOpt = p.bSaveImgOpt;
//    param.proceduremask = p.proceduremask;
    param.liveness_config.occlusion =p.occlusion;
    param.liveness_config.sunglass=p.sunglass;
    param.liveness_config.mask = p.mask;
    param.liveness_config.stable =p.stable;
    param.liveness_config.max_brightness=p.max_brightness;

    param.liveness_config.spooflevel=p.spooflevel;

    param.liveness_config.min_brightness=p.min_brightness;
    param.liveness_config.min_face=p.min_face;
    param.liveness_config.max_face=p.max_face;
    param.liveness_config.hor_margin=p.hor_margin;
    param.liveness_config.ver_margin=p.ver_margin;
    param.liveness_config.yaw_thres=p.yaw_thres;
    param.liveness_config.pitch_thres=p.pitch_thres;
    param.liveness_config.roll_thres=p.roll_thres;
//    param.liveness_config.zero_action=p.zero_action;
    param.liveness_config.occ_thres=p.occ_thres;
    param.liveness_config.clarity_thres=p.clarity_thres;
    param.liveness_config.mask_thres=p.mask_thres;


//    memset(param.imgSavePath, 0, 1024);
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
//    std::string &path = (std::string &) p.imgSavePath.get();
//    memcpy(param.imgSavePath, path.c_str(), path.length());// 保存活体图片的路径，默认当前(仅供POC测试使用)

    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
//    param.bSaveImg = p.bSaveImg;
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);

    cw_face_errcode_t err = cwSetParam(pDet, &param);
    if (err != CW_FACE_OK) {

        LOG_ERROR("cwSetParam error on handle[%d] err=%d", handle, err);

    }

    return err;

}

jint NDKFaceDetTrack::SetFaceJavaObject(JNIEnv *env, jobject thiz, int faceNum,
                                        int op,cw_face_t * cwFaceInfo) {
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
    void *pDet =(void *) env->GetLongField(thiz,m_NativeFaceDetFiled);//获取Native检测器的指针
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
    if (pDet == nullptr) {
        LOG_ERROR("FaceDetectTrack can not find handle[%d]", index);
        return CW_FACE_UNINITIALIZED_ERR;
    }

    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
    cw_face_t *faces = cwFaceInfo;
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
    //人脸坐标信息传到Java层
    if(faceNum){
        jobjectArray faceInfoArray = env->NewObjectArray(faceNum, m_NativeFaceInfoClass, NULL);
        LOGE("faceNum = %d , faceInfoArray=%p", faceNum, &faceInfoArray);
        for (int i = 0; i < faceNum && i < MAX_FACE_NUM; i++) {
            FaceInfo face(env, &faces[i], op);
            face.mapFields();
            jobject faceInfo = env->NewObject(m_NativeFaceInfoClass, m_CFaceInfo);
            face.toJavaObject(env, faceInfo);
            env->SetObjectArrayElement(faceInfoArray, i, faceInfo);
        }
        env->SetObjectField(thiz, m_NativeFaceInfoFiled, faceInfoArray);
    }else{
        env->SetObjectField(thiz, m_NativeFaceInfoFiled, 0);
    }


    if (&faces->reportImage != nullptr && faces->reportImage.data != nullptr) {
//        releaseReportImageMem(pDet);
    }
    return faceNum;
}

jint NDKFaceDetTrack::FaceDetectTrack(JNIEnv *env, jobject thiz,
                                      jbyteArray data, // ???????
                                      jlong time,
                                      jint width, // ??
                                      jint height, // ??
                                      jint format, // ?????
                                      jint angle, // 0 90 180 270
                                      jint mirror,//0 1 2 3
                                      jint op,
                                      jint stageflag//输入图像所处的阶段标志，0：用于脸优判断的语音播报和对齐取景框阶段；1：其他动作阶段
                                      ) {

    void *pDet =(void *) env->GetLongField(thiz,m_NativeFaceDetFiled);//获取Native检测器的指针
    if (pDet == nullptr) {
        return CW_FACE_UNINITIALIZED_ERR;
    }
    if (data == nullptr) {
        return CW_FACE_FORMAT_ERR;
    }

    ByteArray byteArray;
    byteArray.set(env, data);

    cw_image_t image;
    image.angle = (cw_image_angle_t) angle;
    image.data = (char *) byteArray.get();
    image.timestamp = time;
    image.width = width;
    image.height = height;
    image.format = (cw_image_form_t) format;
    image.mirror = (cw_image_mirror_t) mirror;
    image.dataLen = byteArray.size();
    image.coordMap = 0;
    image.stageflag = stageflag;
    int nFace = 0;
    int to_next_stage = 0;
    //Watcher::start();
    cw_face_t faceInfo[MAX_FACE_NUM];
    faceInfo[0].liveness.attack = 0;
    cw_face_errcode_t err = cwFaceDetectTrack(pDet, &image, faceInfo,MAX_FACE_NUM, &nFace, &to_next_stage, op);
    GetLogInfo(env,thiz);
    env->SetIntField(thiz,m_NativeFaceNumFiled,nFace);//设置Java层属性mFaceNum
    if (err != CW_FACE_OK) {
        if(err!=CW_FACE_NO_FACE){
			LOGE("cwFaceDetectTrack err=%d", err);
		}
        return err;
    }
    int ret = SetFaceJavaObject(env, thiz, nFace, op,faceInfo);

    return to_next_stage;
}

jobjectArray  NDKFaceDetTrack::GetLivingImage(JNIEnv *env, jobject thiz) {
    jobjectArray jLivingImgObj = nullptr;
    jclass clazz = env->FindClass("cn/cloudwalk/jni/FaceLivingImg");
    jclass jclsFaceLivingImg = (jclass) env->NewLocalRef(clazz);
    jmethodID facelivingimg_constructor = env->GetMethodID(clazz, "<init>", "()V");
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
    void *pDet =(void *) env->GetLongField(thiz,m_NativeFaceDetFiled);//获取Native检测器的指针
    LOG_ERROR("%p %s %d", pDet, __FUNCTION__, __LINE__);
    if (pDet == nullptr) {
        return NULL;
    }
    cw_living_image_frame_t *livingImg = new cw_living_image_frame_t();
    unsigned char* live_frame_1 = new unsigned char[640*480*3];
    unsigned char* live_frame_2 = new unsigned char[640*480*3];
    unsigned char* live_frame_3 = new unsigned char[640*480*3];
    unsigned char* live_frame_4 = new unsigned char[640*480*3];
    (*livingImg).living_frame_image_1.data = live_frame_1;
    (*livingImg).living_frame_image_2.data = live_frame_2;
    (*livingImg).living_frame_image_3.data = live_frame_3;
    (*livingImg).living_frame_image_4.data = live_frame_4;
    cwGetLivingImageSample(pDet,livingImg);
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);


    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);

    jLivingImgObj = env->NewObjectArray(4,jclsFaceLivingImg,NULL);
    int _num_objs = env->GetArrayLength(jLivingImgObj);
    int result = 0;
    for(int i=0; i<_num_objs; i++) {
        jobject obj_FaceLivingImg = env->NewObject(jclsFaceLivingImg,facelivingimg_constructor);
        if(i==0&& (&(livingImg->living_frame_image_1.data)!=nullptr)){
            FaceLivingImg faceLivingImg1(env,&(*livingImg).living_frame_image_1);
            faceLivingImg1.mapFields();
            LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
            faceLivingImg1.toJavaObject(env, obj_FaceLivingImg);
            env->SetObjectArrayElement(jLivingImgObj, i, obj_FaceLivingImg);
            result ++;
            continue;
        }
        if(i==1&& (&((*livingImg).living_frame_image_2.data)!=nullptr)){
            FaceLivingImg(env,&(*livingImg).living_frame_image_2);
            FaceLivingImg faceLivingImg2(env,&(*livingImg).living_frame_image_2);
            faceLivingImg2.mapFields();
            LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
            faceLivingImg2.toJavaObject(env, obj_FaceLivingImg);
            env->SetObjectArrayElement(jLivingImgObj, i, obj_FaceLivingImg);
            result ++;
            continue;
        }
        if(i==2&& (&((*livingImg).living_frame_image_3.data))!=nullptr){
            FaceLivingImg(env,&(*livingImg).living_frame_image_3);
            FaceLivingImg faceLivingImg3(env,&(*livingImg).living_frame_image_3);
            faceLivingImg3.mapFields();
            LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
            faceLivingImg3.toJavaObject(env, obj_FaceLivingImg);
            env->SetObjectArrayElement(jLivingImgObj, i, obj_FaceLivingImg);
            result ++;
            continue;
        }
        if(i==3 && (&((*livingImg).living_frame_image_4.data)!= nullptr)){
            FaceLivingImg(env,&(*livingImg).living_frame_image_4);
            FaceLivingImg faceLivingImg4(env,&(*livingImg).living_frame_image_4);
            faceLivingImg4.mapFields();
            LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
            faceLivingImg4.toJavaObject(env, obj_FaceLivingImg);
            env->SetObjectArrayElement(jLivingImgObj, i, obj_FaceLivingImg);
            result ++;
            continue;
        }
    }
    if(live_frame_1){
        free(live_frame_1);
        live_frame_1= nullptr;
    }
    if(live_frame_2){
        free((*livingImg).living_frame_image_2.data);
        live_frame_2= nullptr;
    }
    if(live_frame_3){
        free((*livingImg).living_frame_image_3.data);
        live_frame_3= nullptr;
    }
    if(live_frame_4){
        free((*livingImg).living_frame_image_4.data);
        live_frame_4= nullptr;
    }
    delete(livingImg);
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);

    env->DeleteLocalRef(jclsFaceLivingImg);
    return jLivingImgObj;

}


jstring  NDKFaceDetTrack::GetVersionInfo(JNIEnv *env, jobject thiz) {
    char *strVersion;
    void *pDet =(void *) env->GetLongField(thiz,m_NativeFaceDetFiled);//获取Native检测器的指针
    if (pDet == nullptr) {
        LOG_ERROR("GetVersionInfo can not find handle[%d]", handle);
        return env->NewStringUTF("Can not find handle");
    }
    strVersion = (char *) cwGetVersionInfo(pDet);
    LOG_ERROR("GetVersionInfo strVersion=%s", strVersion);

    JavaString testString;
    testString.set(strVersion);
    if(strVersion!= nullptr){
        free(strVersion);
        strVersion = nullptr;
    }
    return testString.toJavaString(env).leak();
}

jint NDKFaceDetTrack::ResetLiving(JNIEnv *env, jobject thiz) {
    LOG_INFO("%s %d", __FUNCTION__, __LINE__);
    void *pDet =(void *) env->GetLongField(thiz,m_NativeFaceDetFiled);//获取Native检测器的指针
    if (pDet == nullptr) {
        LOG_ERROR("ResetLiving can not find handle[%d]", handle);
        return CW_FACE_UNINITIALIZED_ERR;
    }

//    cw_face_errcode_t err = cwResetLiving(pDet);
//    if (err != CW_FACE_OK) {
//
//
//    }
//
//    return err;

}

jint NDKFaceDetTrack::StartFaceSelect(JNIEnv *env, jobject thiz, jint op) {
    LOG_INFO("%s %d", __FUNCTION__, __LINE__);
    void *pDet =(void *) env->GetLongField(thiz,m_NativeFaceDetFiled);//获取Native检测器的指针
    if (pDet == nullptr) {
        LOG_ERROR("cwStartFaceSelect can not find handle[%d]", handle);
        return CW_FACE_UNINITIALIZED_ERR;
    }

//    cw_face_errcode_t err = cwStartFaceSelect(pDet, op);
//    if (err != CW_FACE_OK) {
//
//        LOG_ERROR("cwStartFaceSelect error on handle[%d] err=%d", handle, err);
//
//    }
//
//    return err;

}

jint NDKFaceDetTrack::FinishFaceSelect(JNIEnv *env, jobject thiz,
                                       jobject faceLiving) {
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
    void *pDet =(void *) env->GetLongField(thiz,m_NativeFaceDetFiled);//获取Native检测器的指针
    if (pDet == nullptr) {
        LOG_ERROR("cwFinishFaceSelect can not find handle[%d]", handle);
        return CW_FACE_UNINITIALIZED_ERR;
    }
//    cw_face_living_t facelive;
//    cw_face_errcode_t err = cwFinishFaceSelect(pDet, &facelive);
//    if (err != CW_FACE_OK) {
//
//        LOG_ERROR("cwFinishFaceSelect error on handle[%d] err=%d", handle, err);
//        return -err;
//    }
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
//    FaceLiving face(env, &facelive);
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
//    face.mapFields();
    LOG_ERROR("%s %d", __FUNCTION__, __LINE__);
//    face.toJavaObject(env, faceLiving);
//
//    return err;
}

jint NDKFaceDetTrack::SetLog(JNIEnv *env, jobject thiz,
                                       jboolean openlog){
    void *pDet =(void *) env->GetLongField(thiz,m_NativeFaceDetFiled);//获取Native检测器的指针
    if (pDet == nullptr) {
        LOG_ERROR("SetLog can not find handle[%d]", handle);
        return CW_FACE_UNINITIALIZED_ERR;
    }
    setLog(pDet, openlog);
}

jint NDKFaceDetTrack::GetLogInfo(JNIEnv *env, jobject thiz){

    void *pDet =(void *) env->GetLongField(thiz,m_NativeFaceDetFiled);//获取Native检测器的指针
    if (pDet == nullptr) {
        LOG_ERROR("GetLogInfo can not find handle[%d]", handle);
        return CW_FACE_UNINITIALIZED_ERR;
    }
    int bufflen = 1024;
    char *loginfo = new char[bufflen];
    int ret = getLogInfo(pDet,loginfo,bufflen);
    if(ret ==0){
//        LOGE("kevin GetLogInfo.loginfo= %s",loginfo);
    }
}

jint NDKFaceDetTrack::SaveLogAndImage(JNIEnv *env, jobject thiz, jboolean on, jstring path, jboolean imgOn){
    void *pDet =(void *) env->GetLongField(thiz,m_NativeFaceDetFiled);//获取Native检测器的指针
    if (pDet == nullptr) {
        LOG_ERROR("GetLogInfo can not find handle[%d]", handle);
        return CW_FACE_UNINITIALIZED_ERR;
    }
    JavaString jFilePath(env, path);
    std::string convertedPath = jFilePath.get();
    const char *p = convertedPath.c_str();
    setLogAndSaveImage(pDet, on, (char*)p, imgOn);
}


void NDKFaceDetTrack::initialize(JNIEnv *env) {
    setClass(env);

    addNativeMethod("cwCreateDetectorFromFile", (void *) &CreateDetectorFromFile, kTypeInt,
                    kTypeString, kTypeString, kTypeString, kTypeString, kTypeString, kTypeString,
                    kTypeInt, NULL);
    addNativeMethod("cwReleaseDetector", (void *) &ReleaseDetector, kTypeInt, NULL);
    addNativeMethod("cwResetLivenessTarget", (void *) &ResetLivenessTarget, kTypeInt, NULL);
    addNativeMethod("cwGetVersionInfo", (void *) &GetVersionInfo, kTypeString, NULL);
//    addNativeMethod("cwResetLiving", (void *) &ResetLiving, kTypeInt,
//                    NULL);
    //函数名后的第一个是返回值类型，后面是参数类型，后面的NULL是找到NULL为结尾
    addNativeMethod("cwVerifyBestImg", (void *) &VerifyBestImg, kTypeInt, NULL);//前端活体攻击检验
    FaceParam param;
    addNativeMethod("cwGetParam", (void *) &GetParam, kTypeInt, param.getCanonicalName(),
                    NULL);
    addNativeMethod("cwSetParam", (void *) &SetParam, kTypeInt, param.getCanonicalName(),
                    NULL);

//    addNativeMethod("cwStartFaceSelect", (void *) &StartFaceSelect, kTypeInt, kTypeInt,
//                    NULL);
    addNativeMethod("cwFinishFaceSelect", (void *) &FinishFaceSelect, kTypeInt,
                    "cn/cloudwalk/jni/FaceLiving", NULL);

    addNativeMethod("cwFaceDetectTrack", (void *) &FaceDetectTrack, kTypeInt,
                    kTypeArray(kTypeByte),kTypeLong, kTypeInt, kTypeInt, kTypeInt, kTypeInt, kTypeInt,
                    kTypeInt, kTypeInt, NULL);

    addNativeMethod("setLog", (void *)&SetLog,kTypeInt,kTypeBool,NULL);
    addNativeMethod("getLogInfo", (void *)&GetLogInfo,kTypeInt,NULL);

    addNativeMethod("setLogAndSaveImage", (void *)&SaveLogAndImage, kTypeInt, kTypeBool, kTypeString, kTypeBool, NULL);

    addNativeMethod("cwGetLivingImage", (void *) &GetLivingImage,
                    kTypeObjectArray("cn/cloudwalk/jni/FaceLivingImg"), NULL);
    registerNativeMethods(env);
    jclass clazz = env->FindClass("cn/cloudwalk/jni/FaceDetTrack");
    m_NativeFaceDetFiled = env->GetFieldID(clazz,"mNativeDet","J");
    m_NativeFaceNumFiled = env->GetFieldID(clazz,"mFaceNum","I");
    m_NativeFaceInfoFiled = env->GetFieldID(clazz,"faceInfos","[Lcn/cloudwalk/jni/FaceInfo;");//FaceDetTrack中FaceInfo[] faceInfos字段

    jclass faceInfoClazz = env->FindClass("cn/cloudwalk/jni/FaceInfo");
    m_NativeFaceInfoClass =  (jclass)env->NewGlobalRef(faceInfoClazz);
    m_CFaceInfo = env->GetMethodID(m_NativeFaceInfoClass,"<init>", "()V");

    env->DeleteLocalRef(clazz);
    env->DeleteLocalRef(faceInfoClazz);

}


