/*
 * Copyright (c) 2014 Spotify AB
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

#ifndef __FaceLiving_h__
#define __FaceLiving_h__

#include "CloudWalkSDK.h"
#include "../Algorithm/CWFaceDetTrackDef.h"

using namespace spotify::jni;

/**
 * @brief Basic model-type class
 *
 * This class is a very simple object which holds a few pieces of data. It
 * demonstrates conversion between Java <--> C++ objects.
 */
class EXPORT FaceLiving : public JavaClass {
public:
  /**
   * @brief Basic no-arg constructor
   *
   * If you want to use ClassRegistry::newInstance(), then you must provide
   * a no-arg constructor.
   */
  FaceLiving();
  /**
   * @brief Initialization constructor
   * @param env JNIEnv
   *
   * Normally you shouldn't call this constructor directly, instead it is invoked
   * in JNI_OnLoad and should call initialize() in order to cache any method/field
   * definitions for later.
   */
//  FaceLiving(JNIEnv *env,cw_face_living_t* face);
  ~FaceLiving() {}

  /**
   * @brief Return the class canonical name
   * @return String with JNI canonical name (ie, "com/example/Foo")
   *
   * Use of the MAKE_CANONICAL_NAME macro is recommended here
   */
  const char* getCanonicalName() const {
    return MAKE_CANONICAL_NAME(PACKAGE, FaceLiving);
  }

  /**
   * @brief Initialize class info, which is utilized by ClassRegistry::newInstance
   * @param env JNIEnv
   */
  virtual void initialize(JNIEnv *env);

  /**
   * @brief Configure field mappings
   *
   * Field mappings tell JniHelpers how to copy data between Java <--> C++ instances
   * when invoking toJavaObject() and setJavaObject(). If you do not need to use
   * these methods, then it is not necessary to do any work in this method.
   */
  virtual void mapFields();

public:
	
	int code;
	int iAction;
	
	int	  nkeypt;
	float keyptScore; // 关键点得分
	// face_aligned
	//unsigned char alignedData[CW_ALIGNED_IMAGE_SIZE * CW_ALIGNED_IMAGE_SIZE]; // 图像数据,固定大小SIZE_ALIGNED_IMAGE
	int alignedW; // 宽
	int alignedH; // 高
	int nChannels; // 图像通道
	
	// head_pose
	float pitch; // 抬头、低头,范围-90到90，越大表示越抬头
	float yaw; // 左右转头
	float roll; // 平面内偏头
	FloatArray keypt_x;
 	FloatArray keypt_y;
	ByteArray  alignedData;
};

#endif // __TestObject_h__
