/*
 * Copyright (c) 2014 Spotify AB
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

#include "FaceLivingImg.h"


FaceLivingImg::FaceLivingImg() : JavaClass()
{

}

FaceLivingImg::FaceLivingImg(JNIEnv *env,living_image_frame_t* livingimg) : JavaClass(env)
{
	keyptScore = livingimg->keypoint_score;
	nkeypt = livingimg->nkeypt;
	int iSize = (livingimg->nkeypt >= CW_MAX_FACE_KEYPT_NUM)?CW_MAX_FACE_KEYPT_NUM:livingimg->nkeypt;
	float x[iSize],y[iSize];
	for( int i = 0; i < iSize; i++ )
	{
		x[i] = livingimg->keypoints[i].x;
		y[i] = livingimg->keypoints[i].y;
	}
	keypt_x.set(x,iSize,true);
	keypt_y.set(y,iSize,true);


	pitch = livingimg->pitch;
	yaw = livingimg->yaw;
	roll = livingimg->roll;

	livingImageW = livingimg->width;
	livingImageH = livingimg->height;
	livingImageChannel = livingimg->nChannels;
	livingImageData.set(livingimg->data,livingimg->width*livingimg->height*livingimg->nChannels,true);

	livingTimeStamp = livingimg->timestamp;

  initialize(env);
  // Set up field mappings for the instance. Normally this isn't necessary,
  // since the global instance is typically used for merging into other objects.
  // If you want to actually use the initialized instance in combination with
  // setJavaObject(), you should call this though.
  merge(this);
}

void FaceLivingImg::initialize(JNIEnv *env) {
  setClass(env);
	
	cacheField(env,"keyptScore", kTypeFloat);
	cacheField(env,"nkeypt", kTypeInt);
	cacheField(env,"pointX", kTypeArray(kTypeFloat));
	cacheField(env,"pointY", kTypeArray(kTypeFloat));

	cacheField(env,"pitch", kTypeFloat);
	cacheField(env,"yaw", kTypeFloat);
	cacheField(env,"roll", kTypeFloat);

	
	cacheField(env,"livingImageW", kTypeInt);
	cacheField(env,"livingImageH", kTypeInt);
	cacheField(env,"livingImageChannel", kTypeInt);
	cacheField(env,"livingImageData", kTypeArray(kTypeByte));

	cacheField(env,"livingTimeStamp", kTypeLong);
}

void FaceLivingImg::mapFields() {

	LOG_DEBUG("mapFileds");
	

	mapField("keyptScore", kTypeFloat, &keyptScore);
	mapField("nkeypt", kTypeInt, &nkeypt);
	mapField("pointX", kTypeArray(kTypeFloat), &keypt_x);
	mapField("pointY", kTypeArray(kTypeFloat), &keypt_y);

	mapField("pitch", kTypeFloat, &pitch);
	mapField("yaw", kTypeFloat, &yaw);
	mapField("roll", kTypeFloat, &roll);

	mapField("livingImageW", kTypeInt, &livingImageW);
	mapField("livingImageH", kTypeInt, &livingImageH);
	mapField("livingImageChannel", kTypeInt, &livingImageChannel);
	mapField("livingImageData", kTypeArray(kTypeByte), &livingImageData);

	mapField("livingTimeStamp", kTypeLong, &livingTimeStamp);
}
