/*
 * Copyright (c) 2014 Spotify AB
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

#ifndef __FaceLivingImg_h__
#define __FaceLivingImg_h__

#include "CloudWalkSDK.h"
#include "../Algorithm/CWFaceDetTrackDef.h"

using namespace spotify::jni;

/**
 * @brief Basic model-type class
 *
 * This class is a very simple object which holds a few pieces of data. It
 * demonstrates conversion between Java <--> C++ objects.
 */
class EXPORT FaceLivingImg : public JavaClass {
public:
  /**
   * @brief Basic no-arg constructor
   *
   * If you want to use ClassRegistry::newInstance(), then you must provide
   * a no-arg constructor.
   */
  FaceLivingImg();
  /**
   * @brief Initialization constructor
   * @param env JNIEnv
   *
   * Normally you shouldn't call this constructor directly, instead it is invoked
   * in JNI_OnLoad and should call initialize() in order to cache any method/field
   * definitions for later.
   */
  FaceLivingImg(JNIEnv *env,living_image_frame_t* living_frame);
  ~FaceLivingImg() {}

  /**
   * @brief Return the class canonical name
   * @return String with JNI canonical name (ie, "com/example/Foo")
   *
   * Use of the MAKE_CANONICAL_NAME macro is recommended here
   */
  const char* getCanonicalName() const {
    return MAKE_CANONICAL_NAME(PACKAGE, FaceLivingImg);
  }

  /**
   * @brief Initialize class info, which is utilized by ClassRegistry::newInstance
   * @param env JNIEnv
   */
  virtual void initialize(JNIEnv *env);

  /**
   * @brief Configure field mappings
   *
   * Field mappings tell JniHelpers how to copy data between Java <--> C++ instances
   * when invoking toJavaObject() and setJavaObject(). If you do not need to use
   * these methods, then it is not necessary to do any work in this method.
   */
  virtual void mapFields();

public:
    /** 关键点得分 **/
    float keyptScore; // 关键点得分
    int	  nkeypt;//关键点个数
    /** 关键点x集合 **/
    FloatArray keypt_x;// 64
    /** 关键点y集合 **/
    FloatArray keypt_y;// 64
    float pitch; // 抬头、低头,范围-90到90，越大表示越抬头
    float yaw; // 左右转头
    float roll; // 平面内偏头

    int livingImageW;  //宽
    int livingImageH;   //高
    int livingImageChannel;  //图像通道
    ByteArray livingImageData;   //图像数据
    long livingTimeStamp;    //时间戳

};

#endif // __TestObject_h__
