/*
 * Copyright (c) 2014 Spotify AB
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

#include "FaceParam.h"


FaceParam::FaceParam() : JavaClass()
{

}

FaceParam::FaceParam(JNIEnv *env,cw_face_param_t* para) : JavaClass(env)
{
	if(para)
	{
		roiX = para->roiX;
		roiY = para->roiY;
		maxFaceNumPerImg = para->maxFaceNumPerImg;  //// 每帧最大人脸数，默认20
//		minSize = para->minSize;                    //// 人脸尺寸范围， pc端默认(30 ~ 1000);移动端默认(100 ~ 400)
//		maxSize = para->maxSize;
		roiHeight = para->roiHeight;
		roiWidth = para->roiWidth;
//		globleDetFreq = para->globleDetFreq;
//		minTrackedKeyptScore = para->minTrackedKeyptScore;
		
		
//		imageMode = para->imageMode;
//		//minLivingFrames = para->minLivingFrames;
//		nMissAllowed = para->nMissAllowed;
		perfmonLevel = para->perfmonLevel;
//		nCpuCores = para->nCpuCores;
//		minLivingBrightness = para->minLivingBrightness;
//		bCollectFrmsLiving = para->bCollectFrmsLiving;
//
//		bSaveImg = para->bSaveImg; //是否保存, 默认false(仅供POC测试使用)
//		imgSavePath.set(para->imgSavePath);  // 保存活体图片的路径，默认当前(仅供POC测试使用)
//
//		bSaveImgOpt = para->bSaveImgOpt;
//		proceduremask = para->proceduremask;

		occlusion = para->liveness_config.occlusion;
		stable = para->liveness_config.stable;
		mask = para->liveness_config.mask;
		sunglass = para->liveness_config.sunglass;
		max_brightness=para->liveness_config.max_brightness;

//		zero_action = para->liveness_config.zero_action;
		spooflevel = para->liveness_config.spooflevel;
		min_brightness = para->liveness_config.min_brightness;
		min_face = para->liveness_config.min_face;
		max_face = para->liveness_config.max_face;
		hor_margin = para->liveness_config.hor_margin;
		ver_margin = para->liveness_config.ver_margin;
		yaw_thres = para->liveness_config.yaw_thres;
		pitch_thres = para->liveness_config.pitch_thres;
		roll_thres = para->liveness_config.roll_thres;

		occ_thres = para->liveness_config.occ_thres;
		clarity_thres = para->liveness_config.clarity_thres;
		mask_thres = para->liveness_config.mask_thres;
	}
	
		

initialize(env);
  // Set up field mappings for the instance. Normally this isn't necessary,
  // since the global instance is typically used for merging into other objects.
  // If you want to actually use the initialized instance in combination with
  // setJavaObject(), you should call this though.
  merge(this);
}

void FaceParam::initialize(JNIEnv *env) {
  setClass(env);
	
	cacheField(env, "roiX", kTypeInt);
	cacheField(env, "roiY", kTypeInt);
	cacheField(env, "roiWidth", kTypeInt);
	cacheField(env, "roiHeight", kTypeInt);
//	cacheField(env, "minSize", kTypeInt);
//	cacheField(env, "maxSize", kTypeInt);
	cacheField(env, "maxFaceNumPerImg", kTypeInt);
//	cacheField(env, "globleDetFreq", kTypeInt);
//	cacheField(env, "minTrackedKeyptScore", kTypeFloat);
	
	//cacheField(env, "nMinNeighbors", kTypeInt);
	//cacheField(env, "iSmoothness", kTypeInt);
//	cacheField(env, "imageMode", kTypeInt);
	//cacheField(env, "minLivingFrames", kTypeInt);
//	cacheField(env, "nMissAllowed", kTypeInt);
	cacheField(env, "perfmonLevel", kTypeInt);
//	cacheField(env, "nCpuCores", kTypeInt);
//	cacheField(env,"minLivingBrightness",kTypeFloat);
//	cacheField(env,"bCollectFrmsLiving",kTypeInt);
//
//
//	cacheField(env, "bSaveImg", kTypeInt);
//	cacheField(env, "imgSavePath", kTypeString);
//
//	cacheField(env, "bSaveImgOpt", kTypeInt);
//	cacheField(env, "proceduremask", kTypeInt);

	cacheField(env, "occlusion", kTypeBool);
	cacheField(env, "stable", kTypeBool);
	cacheField(env, "mask", kTypeBool);
	cacheField(env, "sunglass", kTypeBool);

	cacheField(env, "max_brightness", kTypeFloat);
//	cacheField(env, "zero_action", kTypeBool);
	cacheField(env, "spooflevel", kTypeInt);
	cacheField(env, "min_brightness", kTypeFloat);
	cacheField(env, "min_face", kTypeFloat);
	cacheField(env, "max_face", kTypeFloat);
	cacheField(env, "hor_margin", kTypeFloat);
	cacheField(env, "ver_margin", kTypeFloat);
	cacheField(env, "yaw_thres", kTypeFloat);
	cacheField(env, "pitch_thres", kTypeFloat);
	cacheField(env, "roll_thres", kTypeFloat);
	cacheField(env, "occ_thres", kTypeFloat);
	cacheField(env, "clarity_thres", kTypeFloat);
	cacheField(env, "mask_thres", kTypeFloat);

 	cacheConstructor(env);
	
}

void FaceParam::mapFields() {

	LOG_DEBUG("mapFileds");
	mapField("roiX", kTypeInt, &roiX);
	mapField("roiY", kTypeInt, &roiY);
	mapField("roiWidth", kTypeInt, &roiWidth);
	mapField("roiHeight", kTypeInt, &roiHeight);
//	mapField("minSize", kTypeInt, &minSize);
//	mapField("maxSize", kTypeInt, &maxSize);
	mapField("maxFaceNumPerImg", kTypeInt, &maxFaceNumPerImg);
//	mapField("globleDetFreq", kTypeInt, &globleDetFreq);
//	mapField("minTrackedKeyptScore", kTypeFloat, &minTrackedKeyptScore);
	
	//mapField("nMinNeighbors", kTypeInt,&nMinNeighbors);
	//mapField("iSmoothness", kTypeInt,&iSmoothness);
//	mapField("imageMode", kTypeInt,&imageMode);
	//mapField("minLivingFrames", kTypeInt,&minLivingFrames);
//	mapField("nMissAllowed", kTypeInt,&nMissAllowed);
	mapField("perfmonLevel", kTypeInt,&perfmonLevel);
//	mapField("nCpuCores", kTypeInt,&nCpuCores);
//	mapField("minLivingBrightness",kTypeFloat,&minLivingBrightness);
//	mapField("bCollectFrmsLiving",kTypeInt,&bCollectFrmsLiving);
//	mapField("bSaveImg", kTypeInt,&bSaveImg);
//	mapField("imgSavePath", kTypeString,&imgSavePath);


//	mapField("bSaveImgOpt", kTypeInt,&bSaveImgOpt);
//	mapField("proceduremask", kTypeInt,&proceduremask);

	mapField("occlusion", kTypeBool,&occlusion);
	mapField("stable", kTypeBool,&stable);
	mapField("mask", kTypeBool,&mask);
	mapField("sunglass", kTypeBool,&sunglass);
	mapField("max_brightness",kTypeFloat,&max_brightness);
//	mapField("zero_action",kTypeFloat,&zero_action);
	mapField("spooflevel",kTypeInt,&spooflevel);

	mapField("min_brightness",kTypeFloat,&min_brightness);
	mapField("min_face",kTypeFloat,&min_face);
	mapField("max_face",kTypeFloat,&max_face);
	mapField("hor_margin",kTypeFloat,&hor_margin);
	mapField("ver_margin",kTypeFloat,&ver_margin);
	mapField("yaw_thres",kTypeFloat,&yaw_thres);
	mapField("pitch_thres",kTypeFloat,&pitch_thres);
	mapField("roll_thres",kTypeFloat,&roll_thres);

	mapField("occ_thres",kTypeFloat,&occ_thres);
	mapField("clarity_thres",kTypeFloat,&clarity_thres);
	mapField("mask_thres",kTypeFloat,&mask_thres);

}
