/*
 * Copyright (c) 2014 Spotify AB
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

#ifndef __FaceParam_h__
#define __FaceParam_h__

#include "CloudWalkSDK.h"
#include "../Algorithm/CWFaceDetTrackDef.h"

using namespace spotify::jni;

/**
 * @brief Basic model-type class
 *
 * This class is a very simple object which holds a few pieces of data. It
 * demonstrates conversion between Java <--> C++ objects.
 */
class EXPORT FaceParam : public JavaClass {
public:
    /**
     * @brief Basic no-arg constructor
     *
     * If you want to use ClassRegistry::newInstance(), then you must provide
     * a no-arg constructor.
     */
    FaceParam();

    /**
     * @brief Initialization constructor
     * @param env JNIEnv
     *
     * Normally you shouldn't call this constructor directly, instead it is invoked
     * in JNI_OnLoad and should call initialize() in order to cache any method/field
     * definitions for later.
     */
    FaceParam(JNIEnv *env, cw_face_param_t *para = NULL);

    ~FaceParam() {}

    /**
     * @brief Return the class canonical name
     * @return String with JNI canonical name (ie, "com/example/Foo")
     *
     * Use of the MAKE_CANONICAL_NAME macro is recommended here
     */
    const char *getCanonicalName() const {
        return MAKE_CANONICAL_NAME(PACKAGE, FaceParam);
    }

    /**
     * @brief Initialize class info, which is utilized by ClassRegistry::newInstance
     * @param env JNIEnv
     */
    virtual void initialize(JNIEnv *env);

    /**
     * @brief Configure field mappings
     *
     * Field mappings tell JniHelpers how to copy data between Java <--> C++ instances
     * when invoking toJavaObject() and setJavaObject(). If you do not need to use
     * these methods, then it is not necessary to do any work in this method.
     */
    virtual void mapFields();

public:

    int roiX; // roi, ?????????0, 0, 0, 0
    int roiY;
    int roiWidth;
    int roiHeight;
    int maxFaceNumPerImg; // ????????????????20
    int perfmonLevel;

//    bool zero_action;
    int spooflevel;                // Ĭ0ǿ1ǿͨ-1עؾ0
    float min_brightness;        // ׼׶ֵĬ0.15עcw_face_paramеminLivingBrightnessͻûʧЧ
    float max_brightness;        // ׼׶ֵĬ0.9
    float min_face;                // ׼׶СĬ96עcw_face_paramеminSizeͻûʧЧ
    float max_face;                // ׼׶Ĭ300עcw_face_paramеmaxSizeͻûʧЧ
    float hor_margin;            // ׼׶κ߽Ĭ0.15
    float ver_margin;            // ׼׶±߽Ĭ0.15
    float yaw_thres;            // ׼׶yawֵĬ25
    float pitch_thres;            // ׼׶pitchֵĬ25
    float roll_thres;            // ׼׶rollֵĬ25
    bool occlusion;                // ׼׶ǷڵĬtrue
    bool stable;                // ׼׶ǷȶĬfalse
    bool mask;                    // ׼׶Ƿ֣Ĭfalseעcw_face_paramеproceduremaskͻûʧЧ
    bool sunglass;
    float occ_thres;            // ڵģ͵ֵĬ0.95
    float clarity_thres;        // ģ͵ֵĬ0.35
    float mask_thres;

};

#endif // __TestObject_h__
