/*
 * Copyright (c) 2014 Spotify AB
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

#include "FaceReportImg.h"


FaceReportImg::FaceReportImg() : JavaClass()
{

}

FaceReportImg::FaceReportImg(JNIEnv *env,cw_face_report_image_t* face) : JavaClass(env)
{
	
  x = face->faceRect.x;
  y = face->faceRect.y;
  width  = face->faceRect.width;
  height = face->faceRect.height;
  keyptScore = face->keyPts.keyptScore;
  nkeypt = face->keyPts.nkeypt;
  
  pitch = face->headPose.pitch;
  yaw = face->headPose.yaw;
  roll = face->headPose.roll;
  
  opType = face->opType;
  reportImageW = face->width;
  reportImageH = face->height;
  reportImageChannel = face->nChannels;
  reportImageData.set(face->data,face->width*face->height*face->nChannels,true);
	reportTimeStamp=face->timeStamp;
  //这里注意用false是使用的内部引用的方式.
	
	float x[CW_MAX_FACE_KEYPT_NUM],y[CW_MAX_FACE_KEYPT_NUM];
	int iSize = (face->keyPts.nkeypt >= CW_MAX_FACE_KEYPT_NUM)?CW_MAX_FACE_KEYPT_NUM:face->keyPts.nkeypt;
	for( int i = 0; i < iSize; i++ )
	{
		x[i] = face->keyPts.points[i].x;
		y[i] = face->keyPts.points[i].y;
	}
	keypt_x.set(x,iSize,true);
	keypt_y.set(y,iSize,true);

 //copy float arry[]

  initialize(env);
  // Set up field mappings for the instance. Normally this isn't necessary,
  // since the global instance is typically used for merging into other objects.
  // If you want to actually use the initialized instance in combination with
  // setJavaObject(), you should call this though.
  merge(this);
}

void FaceReportImg::initialize(JNIEnv *env) {
  setClass(env);
	
	cacheField(env,"x", kTypeInt);
	cacheField(env,"y", kTypeInt);
	cacheField(env,"width", kTypeInt);
	cacheField(env,"height", kTypeInt);


	cacheField(env,"keyptScore", kTypeFloat);
	cacheField(env,"pointX", kTypeArray(kTypeFloat));
	cacheField(env,"pointY", kTypeArray(kTypeFloat));
	cacheField(env,"nkeypt", kTypeInt);
	
	cacheField(env,"pitch", kTypeFloat);
	cacheField(env,"yaw", kTypeFloat);
	cacheField(env,"roll", kTypeFloat);

	
	cacheField(env,"opType", kTypeInt);
	cacheField(env,"reportImageW", kTypeInt);
	cacheField(env,"reportImageH", kTypeInt);
	cacheField(env,"reportImageChannel", kTypeInt);
	cacheField(env,"reportTimeStamp", kTypeLong);
	cacheField(env,"reportImageData", kTypeArray(kTypeByte));

}

void FaceReportImg::mapFields() {

	LOG_DEBUG("mapFileds");
	
	mapField("x", kTypeInt, &x);
	mapField("y", kTypeInt, &y);
	mapField("width", kTypeInt, &width);
	mapField("height", kTypeInt, &height);


	mapField("keyptScore", kTypeFloat, &keyptScore);
	mapField("pointX", kTypeArray(kTypeFloat), &keypt_x);
	mapField("pointY", kTypeArray(kTypeFloat), &keypt_y);
	mapField("nkeypt", kTypeInt, &nkeypt);
	
	mapField("pitch", kTypeFloat, &pitch);
	mapField("yaw", kTypeFloat, &yaw);
	mapField("roll", kTypeFloat, &roll);

	mapField("opType", kTypeInt, &opType);
	mapField("reportImageW", kTypeInt, &reportImageW);
	mapField("reportImageH", kTypeInt, &reportImageH);
	mapField("reportImageChannel", kTypeInt, &reportImageChannel);
	mapField("reportTimeStamp", kTypeLong, &reportTimeStamp);
	mapField("reportImageData", kTypeArray(kTypeByte), &reportImageData);
}
